@@ -1,13 +1,41 @@
-import compling.nlptoolkit as nlptoolkit
+from compling.nlptoolkit import NLP
 from compling.config import ConfigManager
 from spacy.tokens.token import Token as SpacyToken
+from typing import Dict
 import compling.analysis.lexical.tokenization.token_abc as abctoken
 
+def f2att(function):
+    """On first call, it stores the result of the method in an attribute. <br/>
+       On next ones, the result will be returned without executing the method."""
+    def onCall(self, *args):
+        attribute = function.__name__ + '_'
+        if not hasattr(self, attribute):
+            self.__dict__[attribute] = function(self, *args)
+        return self.__dict__[attribute]
+
+    return onCall
+
 
 class Token(abctoken.Token):
-    def __init__(self, token_id: int, token: SpacyToken, nlp: nlptoolkit.NLP, config: ConfigManager,
-                 sent_id: int, para_id: int, doc_id: int,
-                 sent_pos: int, para_pos: int, doc_pos: int, is_negated: bool, metadata: dict) -> None:
+    """
+    Represents a generic **token**, which is a sequence of characters in the input.
+    A token is identified using a tokenizer, which breaks up the sequence of characters in the input into a sequence
+    of tokens.
+    """
+
+    def __init__(self,
+                 token_id: int,
+                 token: SpacyToken,
+                 nlp: NLP,
+                 config: ConfigManager,
+                 sent_id: int,
+                 para_id: int,
+                 doc_id: int,
+                 sent_pos: int,
+                 para_pos: int,
+                 doc_pos: int,
+                 is_negated: bool,
+                 metadata: dict) -> None:
         """
         Represents a generic **token**, which is a sequence of characters in the input.
         A token is identified using a tokenizer, which breaks up the sequence of characters in the input into a sequence
@@ -16,17 +44,17 @@ class Token(abctoken.Token):
 
 
         Args:
-           token_id (int): unique token identifier.
-           token (SpacyToken): a SpacyToken object.
-           nlp (NLP): a Natural Language Processing object.
-           config (ConfigManager): a Config manager object.
-           sent_id (int): unique sentence identifier. The id of the sentence the token occurs in.
-           para_id (int): unique paragraph identifier. The id of the paragraph the token occurs in.
-           doc_id (int): unique document identifier. The id of the document the token occurs in.
-           sent_pos (int): position index of the token in the sentence.
-           para_pos (int): position index of the token in the paragraph.
-           doc_pos (int): position index of the token in the document.
-           metadata (dict): token metadata as a key-value pairs.
+           token_id (int): Unique token identifier.
+           token (SpacyToken): A SpacyToken object.
+           nlp (NLP): A Natural Language Processing object.
+           config (ConfigManager): A Config manager object.
+           sent_id (int): Unique sentence identifier. The id of the sentence the token occurs in.
+           para_id (int): Unique paragraph identifier. The id of the paragraph the token occurs in.
+           doc_id (int): Unique document identifier. The id of the document the token occurs in.
+           sent_pos (int): Position index of the token in the sentence.
+           para_pos (int): Position index of the token in the paragraph.
+           doc_pos (int): Position index of the token in the document.
+           metadata (dict): Token metadata as a key-value pairs.
 
         Returns:
            None.
@@ -41,12 +69,12 @@ class Token(abctoken.Token):
         # Spacy Token
         self.token = token
 
-        # Avoid exceptions in mongodb
-        self.text_ = self.token.text.replace('.', '\u002E')
-
         # Natural Language Object
         self.nlp = nlp
 
+        # Text
+        self.text_ = self.text()
+
         # position index
         self.sent_pos_ = sent_pos
         self.para_pos_ = para_pos
@@ -63,29 +91,71 @@ class Token(abctoken.Token):
         # add metadata for this token record
         self.add_metadata(metadata)
 
+    @f2att
     def text(self) -> str:
         """Returns the text of the token."""
-        if not self.nlp.is_ngram(self.text_) or self.text_.endswith("__"):
-            return self.text_
+
+        # Avoid exceptions in mongodb
+        text = self.token.text.replace('.', '\u002E')
+
+        if not self.nlp.is_ngram(text) or text.endswith("__"):
+            return text
         else:
-            # ngram: __middle_east__
+            # elif self.nlp.is_ngram(text) and not text.endswith("__"):
+            # e.g. ngram: __middle_east__
             # a token may be: __middle_east__ern
             # so --> __middle_eastern__
-            return "__"+self.text_.replace("__", "")+"__"
+            return "__" + text.replace("__", "") + "__"
+
+    @f2att
+    def sentiment(self) -> Dict[str, float]:
+        """
+        Returns the token polarities. <br/>
+        Each polarity value is between 0 and 1. <br/>
+        _p_ in _[0,1]_
+
+        Returns:
+            Dict[str, float]: Polarities type as keys, sentiment values as values.
+
+                    Example:
+                    token.text="fuck"
+
+                    >> token.sentiment()
+                    {
+                     'pos' : 0.0,
+                     'neg' : 1.0,
+                     'obj' : 0.0
+                    }
+        """
+
+        if self.pos() not in ('ADJ', 'ADV', 'AUX', 'NOUN', 'PROPN', 'VERB'):
+            return {'pos':0.0, 'neg':0.0, 'obj':0.0}
+
+        prefix = "NOT_"
+        text = self.lemma().lower()
+
+        if self.is_negated():
+            text = prefix + text
+
+        return self.nlp.token_sentiment(text)
 
     def shape(self) -> str:
-        """Returns a transformed token version to show orthographic features of the token.
+        """
+        Returns a transformed token version to show orthographic features of the token.
 
         Alphabetic characters are replaced by x or X, and numeric characters are replaced by d, and sequences of the
-        same character are truncated after length 4. For example, "Xxxx" or "dd"."""
+        same character are truncated after length 4. For example, "Xxxx" or "dd".
+        """
         return self.token.shape_
 
     def lower(self) -> str:
         """Returns the lowercase version of the token text."""
         return self.nlp.lower(self.text())
 
+    @f2att
     def stem(self) -> str:
         """Returns the stem of the token text."""
+
         return self.nlp.stem(self.text())
 
     def lemma(self) -> str:
@@ -104,12 +174,16 @@ class Token(abctoken.Token):
         """Returns the language of the token."""
         return self.nlp.iso639
 
+    @f2att
     def is_stopword(self) -> bool:
         """Returns True if the token text is a stopword else False."""
+
         return self.nlp.is_stopword(self.text())
 
+    @f2att
     def is_ngram(self) -> bool:
         """Returns True if the token text is a ngram else False."""
+
         return self.nlp.is_ngram(self.text())
 
     def is_digit(self) -> bool:
@@ -146,6 +220,7 @@ class Token(abctoken.Token):
 
     def add_metadata(self, metadata: dict) -> None:
         """Adds metadata as attributes of the token."""
+
         for k, v in metadata.items():
             if hasattr(self, k + '_'):
                 continue
@@ -153,4 +228,5 @@ class Token(abctoken.Token):
 
     def to_dict(self) -> dict:
         """Converts the token object to a dict record."""
+
         return {k: v for k, v in self.__dict__.items() if k.endswith('_')}
