@@ -53,9 +53,10 @@ class Tokenizer(tokenization_abc.Tokenization):
     that document records. (e.g. title, author, ...).
     """
 
-    def __init__(self, domain_stopwords: List[str] = None) -> None:
+    def __init__(self,
+                 domain_stopwords: List[str] = None) -> None:
         """
-        Creates a new Tokenizer object.
+        **\_\_init\_\_**: Creates a new Tokenizer object.
 
         Args:
             domain_stopwords (List[str], optional, default=None): You can provide a list of arbitrary stopwords specific to your corpus domain.
@@ -70,12 +71,16 @@ class Tokenizer(tokenization_abc.Tokenization):
 
         self.corpus_ngrams = None
 
-    def ngrams2tokens(self, n: Union[int, Iterable[int]], docs_in: Iterable[Dict[str, str]],
+    def ngrams2tokens(self,
+                      n: Union[int, Iterable[int]],
+                      docs_in: Iterable[Dict[str, str]],
                       docs_out: Iterable[Dict[str, str]] = None,
                       pos: Tuple[str, ...] = ("PROPN", "VERB", "NOUN", "ADJ"),
                       corpus_threshold: int = 50,
                       doc_threshold: int = 0,
-                      len_gram: int = 3, include: Tuple[str, ...] = None, replace: bool = False) -> List[Dict[int, str]]:
+                      len_gram: int = 3,
+                      include: Tuple[str, ...] = None,
+                      replace: bool = False) -> Iterable[Dict[int, str]]:
         """
         Calculates frequent n-grams in the corpus. They will be considered as unique tokens during the Tokenization task. <br/>
         Frequent n-grams will be calculated based on the documents in the input stream (docs_in). <br/>
@@ -107,7 +112,7 @@ class Tokenizer(tokenization_abc.Tokenization):
                    ...
 
            corpus_threshold (int, optional, default=50): Filters n-grams that have a corpus frequency greater than corpus_threshold.
-           doc_threshold (int, optional, default=0): Filters n-grams that have in frequency greater than doc_threshold. <br/> The frequency of an n-gram in the corpus is the sum of the frequency of that n-gram in documents where the ngram occurs at least doc_thresold times.
+           doc_threshold (int, optional, default=0): Filters n-grams that have frequency greater than doc_threshold. <br/> The frequency of an n-gram in the corpus is the sum of the frequency of that n-gram in documents where the ngram occurs at least doc_thresold times.
            len_gram (int, optional, default=3): The size of the first and the last token that make up n-grams must be at least 'len_gram'.
 
                    Example:
@@ -124,7 +129,7 @@ class Tokenizer(tokenization_abc.Tokenization):
                    ...
 
            include (Tuple[str, ...], optional, default=None): Include a list of arbitrary n-grams.
-           replace (bool, optional, default=False): If True, replaces a n-gram with its tokens separated by '_'. <br/> Else, concatenates a new token, made merging the n-gram tokens with '_', to the n-gram.
+           replace (bool, optional, default=True): If True, replaces a n-gram with its tokens separated by '_'. <br/> Else, concatenates a new token, made merging the n-gram tokens with '_', to the n-gram.
 
                    Example:
 
@@ -203,15 +208,25 @@ class Tokenizer(tokenization_abc.Tokenization):
 
         for doc in docs_out:
             for ngram in self.corpus_ngrams:
+
+                ngram = ngram.replace('.', '\u002E')
+
                 if replace:
                     doc[self.text_key] = doc[self.text_key].replace(ngram, "__"+ngram.replace(' ', '_')+"__")
                 else:
                     doc[self.text_key] = doc[self.text_key].replace(ngram, ngram + ' __' + ngram.replace(' ', '_')+"__")
+
             yield doc
 
-    def run(self, docs: Iterable[Dict[str, str]], doc_id: int = 0, token_id: int = 0, sent_id: int = 0,
+    def run(self,
+            docs: Iterable[Dict[str, str]],
+            doc_id: int = 0,
+            token_id: int = 0,
+            sent_id: int = 0,
             para_id: int = 0,
-            para_size: int = 3, index_doc: bool = True, index_sent: bool = True,
+            para_size: int = 3,
+            index_doc: bool = True,
+            index_sent: bool = True,
             index_para: bool = True) -> Iterable[Dict[str, dict]]:
         """
         Runs the tokenization for each document of the corpus.
